<?php

namespace backend\modules\theme\controllers;

use common\models\ThemeFiles;
use Yii;
use common\models\Theme;
use backend\modules\theme\models\search\ThemeSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\ArrayHelper;
use common\models\Model;
use yii\httpclient\Response;
use yii\widgets\ActiveForm;
use app\models\UploadForm;
use yii\web\UploadedFile;

/**
 * ThemeController implements the CRUD actions for Theme model.
 */
class ThemeController extends Controller
{
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['post'],
                ],
            ],
        ];
    }

    /**
     * Lists all Theme models.
     * @return mixed
     */
    public function actionIndex()
    {


        $searchModel = new ThemeSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Theme model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Theme model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $this->layout = '@backend/modules/theme/views/theme/layout.php';

        $model = new Theme();
        $model->status = $model::STATUS_ACTIVE;
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
        }
        return $this->render('create', [
            'model' => $model,
        ]);

    }

    /**
     * Updates an existing Theme model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $this->layout = '@backend/modules/theme/views/theme/layout.php';

        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Theme model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Theme model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Theme the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Theme::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
    

    public function actionFiles($id)
    {
        $model = $this->findModel($id);
        $modelFiles = $model->themeFiles;

        if ($model->load(Yii::$app->request->post())) { //&& $model->save()

            $oldIDs = ArrayHelper::map($modelFiles, 'id', 'id');
            $modelFiles = Model::createMultiple(ThemeFiles::classname(), $modelFiles);
            Model::loadMultiple($modelFiles, Yii::$app->request->post());
            $deletedIDs = array_diff($oldIDs, array_filter(ArrayHelper::map($modelFiles, 'id', 'id')));

            // ajax validation
            if (Yii::$app->request->isAjax) {
                Yii::$app->response->format = Response::FORMAT_JSON;
                return ArrayHelper::merge(
                    ActiveForm::validateMultiple($modelFiles),
                    ActiveForm::validate($model)
                );
            }

            // validate all models
            $valid = $model->validate();
            $valid = Model::validateMultiple($modelFiles) && $valid;

            if ($valid) {
                $transaction = \Yii::$app->db->beginTransaction();
                try {
                    if ($flag = $model->save(false)) {
                        if (! empty($deletedIDs)) {
                            ThemeFiles::deleteAll(['id' => $deletedIDs]);
                        }
                        foreach ($modelFiles as $modelFiles) {
                            $modelFiles->customer_id = $model->id;
                            if (! ($flag = $modelFiles->save(false))) {
                                $transaction->rollBack();
                                break;
                            }
                        }
                    }
                    if ($flag) {
                        $transaction->commit();
                        return $this->redirect(['file', 'theme_id' => $model->id]);
                    }
                } catch (Exception $e) {
                    $transaction->rollBack();
                }
            }
        }
        
        $categoryFiles = $this->findCategoryFiles($model->category_id);
        return $this->render('file', [
            'model' => $model,
            'modelFiles' => (empty($modelFiles)) ? [new ThemeFiles()] : $modelFiles,
            'categoryFiles' =>$categoryFiles
        ]);
    }

    public function actionCreatee()
    {
        $model = new Theme();
        $modelFiles = [new ThemeFiles()];
        if ($model->load(Yii::$app->request->post())) {

            $modelFiles = Model::createMultiple(ThemeFiles::classname());
            Model::loadMultiple($modelFiles, Yii::$app->request->post());

            // ajax validation
            if (Yii::$app->request->isAjax) {
                Yii::$app->response->format = Response::FORMAT_JSON;
                return ArrayHelper::merge(
                    ActiveForm::validateMultiple($modelFiles),
                    ActiveForm::validate($model)
                );
            }

            // validate all models
            $valid = $model->validate();
            $valid = Model::validateMultiple($modelFiles) && $valid;

            if ($valid) {
                $transaction = \Yii::$app->db->beginTransaction();
                try {
                    if ($flag = $model->save(false)) {
                        foreach ($modelFiles as $modelFile) {
                            $modelFile->theme_id = $model->id;
                            if (! ($flag = $modelFile->save(false))) {
                                $transaction->rollBack();
                                break;
                            }
                        }
                    }
                    if ($flag) {
                        $transaction->commit();
                        return $this->redirect(['file', 'theme_id' => $model->id]);
                    }
                } catch (Exception $e) {
                    $transaction->rollBack();
                }
            }
        }

        return $this->render('file', [
            'model' => $model,
            'modelFiles' => (empty($modelFiles)) ? [new ThemeFiles()] : $modelFiles
        ]);
    }
    public function actionUpload()
    {
        $model = new UploadForm();

        if (Yii::$app->request->isPost) {
            $model->imageFile = UploadedFile::getInstance($model, 'imageFile');
            if ($model->upload()) {
                echo 'file is uploaded successfully';
                return true;
            }
        }

        return $this->render('view', ['model' => $model]);
    }


    protected function findCategoryFiles($id){
        $fileDetails = Yii::$app->db->createCommand('SELECT * FROM lw_category_files WHERE category_id=' . $id)
            ->queryAll();
        return $fileDetails;
    }
}
